(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

open Types

(******************  Some abbreviations for types ************************)

(* atomic types *)
let tBool = Typeconst "bool";;
let tInt = Typeconst "int";;
let tAlgo = Typeconst "algo";;
let tSAlgo = Typeconst "symalgo";;
let tPubKey = Typeconst "pubkey";;
let tPrivKey = Typeconst "privkey";;
let tSymKey  = Typeconst "symkey";;
let tMessage = Typeconst "message";;

(* composed types *)
let tBoolBool = Typetuple [tBool; tBool];;
let tIntInt = Typetuple [tInt; tInt];;
(* the type of polymorphic comparison operators *)
let tPolyComp =
  Typearrow(Typetuple [Typevar "'a"; Typevar "'a"], Typeconst "bool");;

(***************  the default signature **********************************)
let default_signature = [
  (* Bool *)
  ( "true", tBool);
  ( "false", tBool);
  ( "&&", Typearrow(tBoolBool, tBool));
  ( "||", Typearrow(tBoolBool, tBool));
  ( "not", Typearrow(tBool, tBool));
  ( "=",  tPolyComp);
  ( "<",  tPolyComp);
  ( ">",  tPolyComp);
  ( "=<", tPolyComp);
  ( ">=", tPolyComp);
  ( "!=", tPolyComp);
  (* Int *)
  ( "+", Typearrow(tIntInt, tInt));
  ( "-", Typearrow(tIntInt, tInt));
  (* Lists *)
  ( "::", Typearrow(
      Typetuple [Typevar "'a"; Typeapply("list", [Typevar "'a"])],
      Typeapply("list", [Typevar "'a"])));
  ( "element", Typearrow(
      Typetuple [Typevar "'a"; Typeapply("list", [Typevar "'a"])],
      tBool));
  (* Association Lists *)
  ( ".", Typearrow(
      Typetuple [Typeapply("table", [Typevar "'a"; Typevar "'b"]);
		 Typevar "'a"],
      Typevar "'b"));
  (* Algo *)
  ( "asym", tAlgo);
  ( "rsa", tAlgo);
  ( "sym", tSAlgo);
  ( "des", tSAlgo);
  ( "crypt", Typearrow(Typetuple[tAlgo;tPubKey;tMessage],tMessage));
  ( "sign", Typearrow(Typetuple[tAlgo;tPrivKey;tMessage],tMessage));
  ( "symcrypt", Typearrow(Typetuple[tSAlgo;tSymKey;tMessage],tMessage))
]
