(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(**************************************************************************)
(* Signatures                                                             *)
(**************************************************************************)

open Types

(* a signature is a pair of association lists, each of them associating  *)
(* a type expression to a name. The first component of the pair is the   *)
(* set of decomposable function symbols, and the second component is the *)
(* of hash function symbols.                                             *)

(* a simple_signature is an association list that associates type   *)
(* expression to names.                                             *)
type simple_signature = (string * type_expression) list

(* a signature is a pair of simple_signatures, the first one for the *)
(* decomposable function symbols, the secondf one for the hash       *)
(* function which can not be decomposed.                             *)
type signature = simple_signature * simple_signature

(* print a signature to an out_channel, including the surrounding *)
(* keywords "signature" and "end" when the signature is non-empty *)
val print_signature: out_channel -> signature -> unit

(* (flatten_signature [x1;...;xn] t)  yields [(x1,t);...;(xn,t)] *)
val flatten_signature : 'a list -> type_expression ->
  ('a * type_expression) list

(* (construct_signature (psig,hsig)), where psig is a signature of *)
(* decomposable function symbols and hsig is a signature of hash   *)
(* function symbols, first checks the well-formedness :            *)
(* - all type_expressions must be well-formed                      *)
(* - no function symbol is declared twice                          *)
(* If this is the case it returns the pair (psig',hsig) where      *)
(* psig' is psig extended by the default signature.                *)
exception Exc_signatures of string
val construct_signature:
  signature -> signature

(* extract the list of constants from a signature *)
val list_of_constants: signature -> string list


